%macro binomial_cl(data= ,n= ,x= );
/* This macro calculates estimate of and 95% confidence limits
   for the probability in a binomial distribution with number
   of trials n based on the observation x. The confidence limits
   are calculated using the formulas (7.27) and (7.28).
   Input to the macro is a data set which for each binomial 
   distribution contains the number of trials in the variable
   specified by n= and the observations in a variable specified
   by x=

   Preben Bl[sild and J|rgen Granfeldt 02/06/02                 */

DATA confidence_int;
SET &data;
x=&x;
n=&n;
estimate=x/n;
lower=(x+1.96**2/2-1.96*sqrt(x*(n-x)/n+1.96**2/4))/(n+1.96**2);
upper=(x+1.96**2/2+1.96*sqrt(x*(n-x)/n+1.96**2/4))/(n+1.96**2);
RUN;

TITLE5 'estimate and 95% confidence limits based on n and x';
PROC PRINT DATA=confidence_int NOOBS;
RUN;

PROC datasets nolist;
   DELETE confidence_int;
RUN;
QUIT;

%mend binomial_cl;
