%macro fittest(data= ,observed=observed, expected=expected, d= );

/**********************************************************************/
/*                                                                    */
/* Given a dataset 'data' containing the variables 'observed' and     */
/* 'expected' this macro calculates the -2lnQ and chi-squared test    */
/* statistics and the corresponding observed levels of significance   */
/* for a smooth hypothesis of order d.                                */
/* First danish vesion                                                */
/* Jorn Attermann 14/4/93                                             */
/*                                                                    */
/* English version                                                    */
/* Preben Bl[sild and J|rgen Granfeldt 29/12/00                       */
/*                                                                    */
/*                                                                    */
/**********************************************************************/


data _mact1;
  set &data(keep= &observed &expected);
  dum=1;

data _mact2 _mact3;
  set _mact1;
  by dum;
  retain _2lnQ chi2 observed_tot expected_tot grp;
  observed_tot+&observed;
  expected_tot+&expected;
  diff=&observed-&expected;
  _2lnQ+2*&observed*log(&observed/&expected);
  chi2+(&observed-&expected)**2/&expected;
  grp+1;
  output _mact2;
  if last.dum then do;
    ngrp=grp;
    d=&d;
    if ngrp-1-d>0 then do;
      testQ=1-probchi(_2lnQ, ngrp-1-d);
      testchi2=1-probchi(chi2, ngrp-1-d);
    end;
    output _mact3;
  end;
  label _2lnQ='-2lnQ' chi2='chi-square' observed_tot='total observed'
        expected_tot='total expected' testQ='Prob > -2lnQ'
        testchi2='Prob > chi-square' grp='group'
        ngrp='number of groups' diff="&observed.-&expected" d='d';
  format _2lnQ 6.4 chi2 6.4 testQ 6.4 testchi2 6.4;

title5 'fittest 1';
proc print data=_mact2 noobs label;
  var grp &observed &expected diff;
run;

title5 ' ';
title6 ' ';
title7 'fittest 2';
proc print data=_mact3 noobs label;
  var observed_tot expected_tot ngrp d _2lnQ testQ chi2 testchi2;
run;

proc datasets nolist;
  delete _mact1 _mact2 _mact3;
run;
quit;

%mend fittest;
