%macro hw(x1, x2, x3);

/**********************************************************************/
/*                                                                    */
/* Calculates the -2lnQ and chi-squared test statistics for the       */
/* hypothesis of Hardy-Weinberg equilibrium and the corresponding     */
/* observed levels of significance.                                   */
/* Furthermore, the frequency of A-genes and the frequency of         */
/* the three genotypes are estimated.                                 */
/* Input is the observed values of the three genotypes AA, Aa and aa. */
/*                                                                    */
/* First danish version:                                              */
/* Jorn Attermann 14/4/93                                             */
/* English version:                                                   */
/* Preben Bl[sild and J|rgen Granfeldt 29/12/00;                      */
/*                                                                    */
/**********************************************************************/

data _mact1;
  array x{3};
  x{1}=&x1;
  x{2}=&x2;
  x{3}=&x3;
  n=x{1}+x{2}+x{3};
  p=(2*x{1}+x{2})/(2*n);
  array e{3};
  e{1}=n*p**2;
  e{2}=n*2*p*(1-p);
  e{3}=n*(1-p)**2;
  array a{3};
  array b{3};
  do i=1 to 3;
    a{i}=x{i}*log(x{i}/e{i});
    b{i}=(x{i}-e{i})**2/e{i};
  end;
  _2lnQ=2*(a{1}+a{2}+a{3});
  chi2=b{1}+b{2}+b{3};
  testQ=1-probchi(_2lnQ,1);
  testchi2=1-probchi(chi2,1);
  A_gener=2*x{1}+x{2};
  label n='n' p='p' _2lnQ='-2lnQ' chi2='chi-square'
        testQ='Prob > -2lnQ' testchi2='Prob > chi-square'
        A_gener='number A-genes';
  format p 6.4 _2lnQ 6.4 chi2 6.4 testQ 6.4 testchi2 6.4;

data _mact2;
  set _mact1;
  array x{3} x1-x3;
  array e{3} e1-e3;
  do i=1 to 3;
    obs=x{i};
    forv=e{i};
    diff=obs-forv;
    if i=1 then genotype='AA';
    if i=2 then genotype='Aa';
    if i=3 then genotype='aa';
    output;
  end;
  label obs='observed' forv='expected' diff='observed-expected'
        genotype='genotype';
  format e1-e3 16.2;


title5 'Hardy-Weinberg 1';
proc print data=_mact2 noobs label;
  var genotype obs forv diff;
run;

title5 ' ';
title6 ' ';
title7 'Hardy-Weinberg 2';
proc print data=_mact1 noobs label;
  var n p A_gener _2lnQ testQ chi2 testchi2;

proc datasets nolist;
  delete _mact1 _mact2;
run;
quit;


%mend hw;
