%macro poisson(data= ,obs= ,m=_een);

/*****************************************************************************/
/*                                                                           */
/* Based on the observed frequences in "obs" this macro calculate            */
/* test statistic for the hypothesis lambda(i)=lambda*m(i) in the            */
/* model x(i) ~ po(lambda(i)).                                               */
/* Furthermore, the estimate of lambda is calculated together with           */
/* the 95% confidence interval for this parameter.                           */
/* The weights are given in "m". If "m" is not specified the weights         */
/* are all set to 1, i.e. m(i)=1.                                            */
/*                                                                           */
/* "startcol" gives the position of the first column in the output.          */
/*                                                                           */
/* Former danish versions:                                                   */
/* Jorn Attermann 25-05-93, revised 17-09-93;                                */
/* J|rgen Grandfeldt 23-10-00                                                */
/*                                                                           */                                                                         */
/* First english version:                                                    */
/* Preben Bl[sild and J|rgen Granfeldt 31-12-00                              */
/*                                                                           */
/*****************************************************************************/
%let startcol=4;

option nonotes;

data _mact1(keep=&obs &m m_cum obs_cum _lambda _lambdal _lambdah);
  set &data end=last;
  retain obs_cum m_cum 0;
  _een=1;
  obs_cum+&obs;
  m_cum+&m;
  if last then do;
    call symput('obs_cum', obs_cum);
    call symput('m_cum', m_cum);
    _lambda=obs_cum/m_cum;
    _lambdal=(obs_cum+.5*1.96**2-1.96*sqrt(obs_cum+.25*1.96**2))/m_cum;
    _lambdah=(obs_cum+.5*1.96**2+1.96*sqrt(obs_cum+.25*1.96**2))/m_cum;
  end;
run;

data _mact2;
  set _mact1 end=last;
  retain _2lnQ _chi2 forv_cum 0;
  _forv=&m*&obs_cum/&m_cum;
  forv_cum+_forv;
  _2lnQ+2*&obs*log(&obs/_forv);
  _chi2+(&obs-_forv)**2/_forv;
  if last then do;
    call symput('n',left(_n_));
    _logtest=1-probchi(_2lnQ,_n_-1);
    _chitest=1-probchi(_chi2,_n_-1);
  end;
run;

/* Print the calculated numbers;*/

%let mpos=%eval(17+&startcol-%length(&m)+1);
%if &m=_een %then %let obspos=%eval(17+&startcol-%length(&obs)+1);
%else %let obspos=%eval(27+&startcol-%length(&obs)+1);

data _null_;
file print;
set _mact2 end=last;
if _n_=1 then do;
  put / @15+&startcol 'Macro "POISSON"';
  put / @&startcol "Dataset:          %upcase(&data)";
  put  @&startcol "Observations:     %upcase(&obs)";
%if &m=_een %then %do;
  put @&startcol 'Weights:          All weights are set to 1';
%end;
%else %do;
  put  @&startcol "Weights:          %upcase(&m)";
%end;
  put  @&startcol "Number of groups: &n";
%if &m=_een %then %do;
  put / / @&startcol "group" @&obspos "%upcase(&obs)" @22+&startcol 'expected';
  put @&startcol '-------------------------------';
end;
put @&startcol _n_ 6.0 @10+&startcol &obs 8.0 @22+&startcol _forv 8.2;
if last then do;
  put @&startcol '-------------------------------';
  put @&startcol+3 'sum' @10+&startcol obs_cum 8.0 @22+&startcol forv_cum 8.2;
%end;
%else %do;
  put / / @&startcol "group" @&mpos "%upcase(&m)" @&obspos "%upcase(&obs)" @32+&startcol 'expected';
  put @&startcol '-----------------------------------------';
end;
put @&startcol _n_ 6.0 @10+&startcol &m 8.0 @20+&startcol &obs 8.0 @32+&startcol _forv 8.2;
if last then do;
  put @&startcol '-----------------------------------------';
  put @&startcol+3 'sum' @10+&startcol m_cum 8.0 @20+&startcol obs_cum 8.0 @32+&startcol forv_cum 8.2;
%end;
  put / / @&startcol 'Test of the hypothesis of proportional parameters:';
  put / @&startcol 'statistic' @12+&startcol '   value' @26+&startcol 'Prob > value';
  put @&startcol '-------------------------------------------';
  put @&startcol '-2lnQ' @12+&startcol _2lnq 8.4 @26+&startcol _logtest 12.4;
  put @&startcol 'chi-square' @12+&startcol _chi2 8.4 @26+&startcol _chitest 12.4;
  put / / @&startcol 'Estimate and 95% confidence interval for lambda';
  put @&startcol 'under the hypothesis of proportional parameters:';
  put / @&startcol 'estimate' @11+&startcol 'lower limit' @28+&startcol 'upper limit';
  put @&startcol '-----------------------------------------';
  put @&startcol _lambda 7.4 @11+&startcol _lambdal 10.4 @28+&startcol _lambdah 10.4;
end;
run;


proc datasets nolist;
  delete _mact1 _mact2;
run;
quit;

option notes;


%mend poisson;
